# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V3::Enums::PricePlaceholderFieldEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED              => "UNSPECIFIED",
  UNKNOWN                  => "UNKNOWN",
  TYPE                     => "TYPE",
  PRICE_QUALIFIER          => "PRICE_QUALIFIER",
  TRACKING_TEMPLATE        => "TRACKING_TEMPLATE",
  LANGUAGE                 => "LANGUAGE",
  FINAL_URL_SUFFIX         => "FINAL_URL_SUFFIX",
  ITEM_1_HEADER            => "ITEM_1_HEADER",
  ITEM_1_DESCRIPTION       => "ITEM_1_DESCRIPTION",
  ITEM_1_PRICE             => "ITEM_1_PRICE",
  ITEM_1_UNIT              => "ITEM_1_UNIT",
  ITEM_1_FINAL_URLS        => "ITEM_1_FINAL_URLS",
  ITEM_1_FINAL_MOBILE_URLS => "ITEM_1_FINAL_MOBILE_URLS",
  ITEM_2_HEADER            => "ITEM_2_HEADER",
  ITEM_2_DESCRIPTION       => "ITEM_2_DESCRIPTION",
  ITEM_2_PRICE             => "ITEM_2_PRICE",
  ITEM_2_UNIT              => "ITEM_2_UNIT",
  ITEM_2_FINAL_URLS        => "ITEM_2_FINAL_URLS",
  ITEM_2_FINAL_MOBILE_URLS => "ITEM_2_FINAL_MOBILE_URLS",
  ITEM_3_HEADER            => "ITEM_3_HEADER",
  ITEM_3_DESCRIPTION       => "ITEM_3_DESCRIPTION",
  ITEM_3_PRICE             => "ITEM_3_PRICE",
  ITEM_3_UNIT              => "ITEM_3_UNIT",
  ITEM_3_FINAL_URLS        => "ITEM_3_FINAL_URLS",
  ITEM_3_FINAL_MOBILE_URLS => "ITEM_3_FINAL_MOBILE_URLS",
  ITEM_4_HEADER            => "ITEM_4_HEADER",
  ITEM_4_DESCRIPTION       => "ITEM_4_DESCRIPTION",
  ITEM_4_PRICE             => "ITEM_4_PRICE",
  ITEM_4_UNIT              => "ITEM_4_UNIT",
  ITEM_4_FINAL_URLS        => "ITEM_4_FINAL_URLS",
  ITEM_4_FINAL_MOBILE_URLS => "ITEM_4_FINAL_MOBILE_URLS",
  ITEM_5_HEADER            => "ITEM_5_HEADER",
  ITEM_5_DESCRIPTION       => "ITEM_5_DESCRIPTION",
  ITEM_5_PRICE             => "ITEM_5_PRICE",
  ITEM_5_UNIT              => "ITEM_5_UNIT",
  ITEM_5_FINAL_URLS        => "ITEM_5_FINAL_URLS",
  ITEM_5_FINAL_MOBILE_URLS => "ITEM_5_FINAL_MOBILE_URLS",
  ITEM_6_HEADER            => "ITEM_6_HEADER",
  ITEM_6_DESCRIPTION       => "ITEM_6_DESCRIPTION",
  ITEM_6_PRICE             => "ITEM_6_PRICE",
  ITEM_6_UNIT              => "ITEM_6_UNIT",
  ITEM_6_FINAL_URLS        => "ITEM_6_FINAL_URLS",
  ITEM_6_FINAL_MOBILE_URLS => "ITEM_6_FINAL_MOBILE_URLS",
  ITEM_7_HEADER            => "ITEM_7_HEADER",
  ITEM_7_DESCRIPTION       => "ITEM_7_DESCRIPTION",
  ITEM_7_PRICE             => "ITEM_7_PRICE",
  ITEM_7_UNIT              => "ITEM_7_UNIT",
  ITEM_7_FINAL_URLS        => "ITEM_7_FINAL_URLS",
  ITEM_7_FINAL_MOBILE_URLS => "ITEM_7_FINAL_MOBILE_URLS",
  ITEM_8_HEADER            => "ITEM_8_HEADER",
  ITEM_8_DESCRIPTION       => "ITEM_8_DESCRIPTION",
  ITEM_8_PRICE             => "ITEM_8_PRICE",
  ITEM_8_UNIT              => "ITEM_8_UNIT",
  ITEM_8_FINAL_URLS        => "ITEM_8_FINAL_URLS",
  ITEM_8_FINAL_MOBILE_URLS => "ITEM_8_FINAL_MOBILE_URLS"
];

1;
