# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V6::Enums::FeedItemQualityDisapprovalReasonEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                        => "UNSPECIFIED",
  UNKNOWN                            => "UNKNOWN",
  PRICE_TABLE_REPETITIVE_HEADERS     => "PRICE_TABLE_REPETITIVE_HEADERS",
  PRICE_TABLE_REPETITIVE_DESCRIPTION => "PRICE_TABLE_REPETITIVE_DESCRIPTION",
  PRICE_TABLE_INCONSISTENT_ROWS      => "PRICE_TABLE_INCONSISTENT_ROWS",
  PRICE_DESCRIPTION_HAS_PRICE_QUALIFIERS =>
    "PRICE_DESCRIPTION_HAS_PRICE_QUALIFIERS",
  PRICE_UNSUPPORTED_LANGUAGE => "PRICE_UNSUPPORTED_LANGUAGE",
  PRICE_TABLE_ROW_HEADER_TABLE_TYPE_MISMATCH =>
    "PRICE_TABLE_ROW_HEADER_TABLE_TYPE_MISMATCH",
  PRICE_TABLE_ROW_HEADER_HAS_PROMOTIONAL_TEXT =>
    "PRICE_TABLE_ROW_HEADER_HAS_PROMOTIONAL_TEXT",
  PRICE_TABLE_ROW_DESCRIPTION_NOT_RELEVANT =>
    "PRICE_TABLE_ROW_DESCRIPTION_NOT_RELEVANT",
  PRICE_TABLE_ROW_DESCRIPTION_HAS_PROMOTIONAL_TEXT =>
    "PRICE_TABLE_ROW_DESCRIPTION_HAS_PROMOTIONAL_TEXT",
  PRICE_TABLE_ROW_HEADER_DESCRIPTION_REPETITIVE =>
    "PRICE_TABLE_ROW_HEADER_DESCRIPTION_REPETITIVE",
  PRICE_TABLE_ROW_UNRATEABLE    => "PRICE_TABLE_ROW_UNRATEABLE",
  PRICE_TABLE_ROW_PRICE_INVALID => "PRICE_TABLE_ROW_PRICE_INVALID",
  PRICE_TABLE_ROW_URL_INVALID   => "PRICE_TABLE_ROW_URL_INVALID",
  PRICE_HEADER_OR_DESCRIPTION_HAS_PRICE =>
    "PRICE_HEADER_OR_DESCRIPTION_HAS_PRICE",
  STRUCTURED_SNIPPETS_HEADER_POLICY_VIOLATED =>
    "STRUCTURED_SNIPPETS_HEADER_POLICY_VIOLATED",
  STRUCTURED_SNIPPETS_REPEATED_VALUES => "STRUCTURED_SNIPPETS_REPEATED_VALUES",
  STRUCTURED_SNIPPETS_EDITORIAL_GUIDELINES =>
    "STRUCTURED_SNIPPETS_EDITORIAL_GUIDELINES",
  STRUCTURED_SNIPPETS_HAS_PROMOTIONAL_TEXT =>
    "STRUCTURED_SNIPPETS_HAS_PROMOTIONAL_TEXT"
];

1;
